/* ========================================================================= */
/* HELPERS ================================================================= */
/* ========================================================================= */

trimInput = function(value) {
	return value.replace(/^\s*|\s*$/g, '');
};

/* ========================================================================= */
/* FUNCTIONS =============================================================== */
/* ========================================================================= */

var siteHandler = {
	/**
	 * Carga un template con los datos que recibe como parámetro.
	 *
	 * @param  {String}   name    	- Nombre del fichero .hbs donde se encuentra el template
	 * @param  {String}   where   	- Selector CSS donde vamos a insertar el template
	 * @param  {Object}	  context	- Objeto con los datos que va a utilizar el template
	 * @param  {Boolean}  append  	- Si es true inserta el template al final de un contenedor para no borrar su contenido
	 * @param  {Function} callback  - Función de callback
	 */
	loadTemplate: function(name, where, context, append, func) {
		var location = where || '#' + name,
			data = context,
			template = JST['app/templates/' + name + '.hbs'],
			html = template(data),
			callback = func;

		if (append)
			$(location).append(html);
		else
			document.querySelector(location).innerHTML = html;
		if (func)
				callback();
	},
	/**
	 * Establece los eventos de la app
	 */
	setEvents: {
		// mostramos y activamos los eventos del botón del menu (header)
		activeMenu: function(e){
			$('#gridIcon').off('click').on('click', function(event) {
				event.preventDefault();
				/* Act on the event */
				// esta clase oculta el grid, así que la quitamos cuando queramos mostrarlo
				$('#mainWrapper').removeClass('hide-grid');
				// ocultamos los botones de la cabecera (boton del grid + boton de la ficha)
				$('#curtainIcon, #gridIcon, #mapIcon').hide();
				$('#site-canvas').removeClass('overflow-hidden');
			});
		},
		// activamos los botones de la sección Más
		activeMas: function(e){
			// boton del gráfico del tamaño
			$('.btn-1').off('click').on('click', function(){
				var elements = $('.s-e');
				//se añade la clase playing para "bloquear" el evento.
				if (!$('.btn-1').hasClass('playing')){
					$('.btn-1 .fa').toggleClass('fa-plus-circle fa-minus-circle');
					$('.btn-1').addClass('playing');
					$('.size-wrapper p').show();
					//ocultamos todos los elementos del size (size-elements + size-labels)
					$('.s-e, .s-l').hide();
					_.each(elements, function(num, index, list){
						// a cada elemento de la gráfica le asignamos un timeOut para mostrarlos uno a uno
						setTimeout(function() {
						    $('.size-element-'+(index+1)).show();
						    //en el ultimo aplicamos un setTimout diferente
						    if (index+1 >= list.length){
								setTimeout(function() {
									$('.btn-1').removeClass('playing');
									$('.btn-1 .fa').toggleClass('fa-plus-circle fa-minus-circle');
								}, 1000);
							}
						}, 1000*(index+1));
					});
				}
			});

			//sexo comun
			$('.btn-2').off('click').on('click', function(event){
				event.preventDefault();
				$('.btn-2 .fa').toggleClass('fa-plus-circle fa-minus-circle');
				$('.genre-icon-2-1, .genre-icon-2-2, .genre-section.common p').toggle();
			});
			//sexo canario
			$('.btn-3').off('click').on('click', function(event){
				event.preventDefault();
				$('.btn-3 .fa').toggleClass('fa-plus-circle fa-minus-circle');
				$('.genre-icon-3-1, .genre-icon-3-2, .genre-section.common-2 p').toggle();
			});

			//fisinomia
			$('.btn-4').off('click').on('click', function(event){
				event.preventDefault();
				if (!$('.btn-4').hasClass('playing')){
					$('.btn-4').addClass('playing');
					var elements = $('.f-e');
					_.each(elements, function(num, index, list){
						$(num).addClass('animated fadeIn');
						$('.'+$(num).data('label')).addClass('animated fadeIn');
						setTimeout(function() {
							$('.f-e, .fisio-label').hide();
							$(num).show();
							$('.'+$(num).data('label')).show();
							if (index+1 >= list.length){
								$('.btn-4').removeClass('playing');
								setTimeout( function(){
									//$('.f-e, .fisio-label').fadeOut();
								}, 1000);
							}
						}, 2000*index+1);
					});
				}
			});

			$('.btn-reproduccion').off('click').on('click', function(event) {
				event.preventDefault();
				$('.eggs-pic-reproduccion').toggle();
				$(this).find('.fa').toggleClass('fa-plus-circle fa-minus-circle');
			});

			$('.btn-puesta').off('click').on('click', function(event) {
				event.preventDefault();
				$('.eggs-pic-puesta').toggle();
				$(this).find('.fa').toggleClass('fa-plus-circle fa-minus-circle');
			});
		},
		// activamos el mapa
		activeMap: function(e){
			L.mapbox.accessToken = 'pk.eyJ1Ijoiam9zZW5tYXIiLCJhIjoiRTBnY2plZyJ9.16b9pM_DfVdunAs6jZmE-A';
			var map = L.mapbox.map('map', 'josenmar.39411859', {worldCopyJump: true});

			// load feature layer from geojson
			var myLayer = L.mapbox.featureLayer().loadURL('js/geojson/data.geojson').addTo(map);

			// load scale indicator to map (left bottom)
			L.control.scale().addTo(map);

			// Set a custom icon on each marker based on feature properties.
			myLayer.on('layeradd', function(e) {
			    var marker = e.layer,
			        feature = marker.feature;

			    marker.setIcon(L.icon(feature.properties.icon));
			});

			$('#mapIcon').off('click').on('click', function(event) {
				event.preventDefault();

				$('#mapLabelWrapper').toggleClass('shown');
			});
		},
		// activamos la pantalla principal (caracteristicas + ficha)
		activePrincipal: function(e){
			// reiniciamos eventos
			$('.state-button, .area, #curtainIcon').off('click');
			// los tres iconos de la ficha (circulares)
			$('.state-button').on('click', function() {
				var $icons = $(this);

				$icons.children('.fa').toggleClass('fa-plus-circle fa-minus-circle');
				$icons.toggleClass('showed');
				$icons.parent().find('.state-hidden-info').toggle();
				$icons.parents('.state').toggleClass('active');
			});

			// iterate through area objects to asign click event for modals
			_.each($('.area'), function(area, index) {
				 $(area).off('click').on('click', function(event) {
				 	event.preventDefault();
				 	// detect if mobile or tablet (could be better because there is some issues about window width in differents browsers)
				 	// if mobile, modals.. if not, custom tooltips
				 	if (window.innerWidth < 768) {
				 		siteHandler.loadTemplate($(area).data('area'),'#nucleusModal .modal-body');
						$('#nucleusModal').modal();
				 	}
				 	else if (window.innerWidth >= 768) {
				 		tooltip = $('#'+$(area).data('tooltip'));
				 		if (tooltip.is(':visible')){
				 			$('.area-tooltip').hide();
				 			//tooltip.hide();
				 		}
				 		else{
				 			$('.area-tooltip').hide();
				 			tooltip.show();
				 			tooltip.off('click').on('click', function(event) {
					 			event.preventDefault();
					 			/* Act on the event */
					 			$(this).hide();
					 		});
				 		}
				 	}
				 });
			});

			// show/hide Ficha
			$('#curtainIcon').off('click').on('click', function(event) {
				event.preventDefault();
				/* Act on the event */
				$('#curtainIcon .fa').toggleClass('fa-arrow-up fa-arrow-down');
				$('.curtain-wrapper').toggleClass('hide-curtain');
			});

			// show/hide graph text
			$('.active-graph').off('click').on('click', function(event) {
				event.preventDefault();

				$(this).find('.fa').toggleClass('fa-plus-circle fa-minus-circle');
				$(this).parent().toggleClass('active');
			});

			$('.btn-sound').off('click').on('click', function(event) {
				siteHandler.loadTemplate('sound','#soundModal .modal-body');
						$('#soundModal').modal();
			});
			$('#soundModal').on('show.bs.modal', function (e) {
				plyr.setup({
					volume:1,
					seekTime:5
				});
				document.querySelectorAll(".player")[0].plyr.play();
			})
			$('#soundModal').on('hide.bs.modal', function (e) {
				document.querySelectorAll(".player")[0].plyr.pause();
			})
		}
	},
	// loader screen when a grid element is clicked
	setLoader: function(){
		var pageWrap = document.getElementById( 'mainWrapper' ),
			triggerLoading = [].slice.call( pageWrap.querySelectorAll( '.element' ) ),
			loader = new SVGLoader( document.getElementById( 'loader' ), { speedIn : 500, easingIn : mina.easeinout } );

		function init() {
			triggerLoading.forEach( function( trigger ) {
				//each grid element event
				trigger.addEventListener( 'click', function( ev ) {
					ev.preventDefault();
					//get data info
					pageToLoad = $(this).data('page'); //page (which page has to load)
					pageColor = $(this).data('color'); //color (color of the loading screen)
					pagePartial = $(this).data('partial'); //partial (load page without curtain)
					// set color class to loader
					$('#loader').addClass(pageColor);
					// show it
					loader.show();
					// after some time hide loader
					setTimeout( function() {
						// hide grid
						siteHandler.toggleContent();
						// change page here
						siteHandler.loadTemplate(pageToLoad,'#main');
						siteHandler.activePage(pageToLoad);
						// if pagePartial, hide curtain!
						if (pagePartial !== undefined){
							$('#curtainIcon .fa').removeClass('fa-arrow-up').addClass('fa-arrow-down');
							$('.curtain-wrapper').toggleClass('hide-curtain');
						}else {
							$('#curtainIcon .fa').removeClass('fa-arrow-down').addClass('fa-arrow-up');
						}
						// grid icon has to appear
						$('#gridIcon').show();
						// hide loader page
						loader.hide();
						setTimeout( function(){
							//at the end, we remove the color for the next time
							$('#loader').removeClass(pageColor);
						}, 1000);
					}, 1000 );
				} );
			} );
		}

		init();
	},
	activePage: function(which, e){
		switch(which){
			case 'principal':
				//activamos eventos para la zona principal
				siteHandler.loadPage(which); //scripts
				siteHandler.setEvents.activePrincipal(); //events
				break;
			case 'map':
				//activamos eventos para la zona del mapa
				siteHandler.loadPage(which);
				siteHandler.setEvents.activeMap();
				break;
			case 'mas':
				//activamos eventos para la zona de conoce mas
				siteHandler.setEvents.activeMas();
				break;
			case 'default':
				//activamos eventos para la zona
				console.log('default');
				break;
		}
	},
	loadPage: function(which, e){
		switch(which){
			case 'principal':
				//ejecutamos el código necesario para la zona principal
				$('#curtainIcon').show();
				break;
			case 'map':
				//ejecutamos el código necesario para la zona del mapa
				$('#mapIcon').show();
				$('#site-canvas').addClass('overflow-hidden');
				break;
			case 'mas':
				//ejecutamos el código necesario para la zona de conoce mas
				break;
			case 'default':
				break;
		}
	},
	toggleContent: function(){
		//hide-grid esconde #grid y muestra #site-canvas
		$('#mainWrapper').toggleClass('hide-grid');
	},
	init: function() {
		//load template layout (header, grid, and site-canvas)
		this.loadTemplate('layout', '#mainWrapper', null, false, function(){
			var $container = $('#container');
			// initialize masonry
			$container.masonry({
				columnWidth: '.grid-sizer',
				itemSelector: '.element'
			});
			$container.masonry('bindResize'); //bind event for resize (fluid)
			siteHandler.setLoader(); //grid events for each element
			siteHandler.setEvents.activeMenu(); //menu button event
			$('#gridIcon').hide(); //hide grid button
		});
		
		(function(d,p){
		    var a=new XMLHttpRequest(),
		        b=d.body;
		    a.open("GET",p,!0);
		    a.send();
		    a.onload=function(){
		        var c=d.createElement("div");
		        c.style.display="none";
		        c.innerHTML=a.responseText;
		        b.insertBefore(c,b.childNodes[0])
		    }
		})(document,"images/sprite.svg");
	}
};

siteHandler.init();
